@@ -73,40 +73,47 @@ controllers.controller("colorsController", function($scope, $http) {
 
     $scope.changeSelectedTextColor = function(color) {
         $scope.selectedColorScheme[$scope.selectedColorSetting] = color;
+        $scope.selectedColorScheme["colordata-" + $scope.selectedColorSetting].color = color;
         $scope.noteThemeChanged();
     }
 
     $scope.sampleTerminalBackgroundColors = ['white', '#' + solarized.base3, '#300', '#003', '#' + solarized.base03, '#232323', '#'+nord.nord0, 'black'];
 
     /* Array of FishColorSchemes */
-    $scope.colorSchemes = [
-        color_scheme_fish_default,
-        color_scheme_ayu_light,
-        color_scheme_ayu_dark,
-        color_scheme_ayu_mirage,
-        color_scheme_solarized_light,
-        color_scheme_solarized_dark,
-        color_scheme_tomorrow,
-        color_scheme_tomorrow_night,
-        color_scheme_tomorrow_night_bright,
-        color_scheme_nord,
-        color_scheme_base16_default_dark,
-        color_scheme_base16_default_light,
-        color_scheme_base16_eighties
-    ];
-    for (var i=0; i < additional_color_schemes.length; i++)
-        $scope.colorSchemes.push(additional_color_schemes[i])
+    $scope.colorSchemes = [];
 
+    isValidColor = function(col) {
+        if (col == "normal") return true;
+        var s = new Option().style;
+        s.color = col;
+        return !!s.color;
+    }
 
-    $scope.getCurrentTheme = function() {
+    $scope.getThemes = function() {
         $http.get("colors/").then(function(arg) {
-            var currentScheme = { "name": "Current", "colors":[], "preferred_background": "black" };
-            var data = arg.data
-            for (var i in data) {
-                currentScheme[data[i].name] = data[i].color;
+            for (var scheme of arg.data) {
+                var currentScheme = { "name": "Current", "colors":[], "preferred_background": "black" };
+                currentScheme["name"] = scheme["theme"];
+                if (scheme["name"]) currentScheme["name"] = scheme["name"];
+                var data = scheme["colors"];
+                if (scheme["preferred_background"]) {
+                    if (isValidColor(scheme["preferred_background"])) {
+                        currentScheme["preferred_background"] = scheme["preferred_background"];
+                    }
+                }
+                if (scheme["url"]) currentScheme["url"] = scheme["url"];
+
+                for (var i in data) {
+                    currentScheme[data[i].name] = interpret_color(data[i].color).replace(/#/, '');
+                    // HACK: For some reason the colors array is cleared later
+                    // So we cheesily encode the actual objects as colordata-, so we can send them.
+                    // TODO: We should switch to keeping the objects, and also displaying them
+                    // with underlines and such.
+                    currentScheme["colordata-" + data[i].name] = data[i];
+                }
+                $scope.colorSchemes.push(currentScheme);
             }
-            $scope.colorSchemes.splice(0, 0, currentScheme);
-            $scope.changeSelectedColorScheme(currentScheme);
+            $scope.changeSelectedColorScheme($scope.colorSchemes[0]);
         })};
 
 	$scope.saveThemeButtonTitle = "Set Theme";
@@ -137,40 +144,55 @@ controllers.controller("colorsController", function($scope, $http) {
                             "user",
                             "host",
                             "cancel",
+                            // Cheesy hardcoded variable names ahoy!
+                            // These are all the pager vars,
+                            // we should really just save all these in a dictionary.
+                            "fish_pager_color_background",
+                            "fish_pager_color_prefix",
+                            "fish_pager_color_progress",
                             "fish_pager_color_completion",
                             "fish_pager_color_description",
-                            "fish_pager_color_prefix",
-                            "fish_pager_color_progress"
+                            "fish_pager_color_selected_background",
+                            "fish_pager_color_selected_prefix",
+                            "fish_pager_color_selected_completion",
+                            "fish_pager_color_selected_description",
+                            // TODO: Setting these to empty currently makes them weird. Figure out why!
+                            /*
+                            "fish_pager_color_secondary_background",
+                            "fish_pager_color_secondary_prefix",
+                            "fish_pager_color_secondary_completion",
+                            "fish_pager_color_secondary_description",
+                            */
                            ];
         var remaining = settingNames.length;
-        for (name of settingNames) {
+        var postdata = {
+            "theme" : $scope.selectedColorScheme["name"],
+            "colors": [],
+        }
+        for (var name of settingNames) {
             var selected;
+            var realname = "colordata-" + name;
             // Skip colors undefined in the current theme
             // js is dumb - the empty string is false,
             // but we want that to mean unsetting a var.
-            if (!$scope.selectedColorScheme[name] && $scope.selectedColorScheme[name] !== '') {
-                // Fall back onto the defaut colorscheme.
-                selected = color_scheme_fish_default[name];
-                if (!selected && selected !== '') {
-                    selected = '';
-                }
+            if (!$scope.selectedColorScheme[realname] && $scope.selectedColorScheme[realname] !== '') {
+                continue;
             } else {
-                selected = $scope.selectedColorScheme[name];
+                selected = $scope.selectedColorScheme[realname];
             }
-            var postData = "what=" + name + "&color=" + selected + "&background_color=&bold=&underline=&dim=&reverse=&italics=";
-            $http.post("set_color/", postData, { headers: {'Content-Type': 'application/x-www-form-urlencoded'} }).then(function(arg) {
-            	if (arg.status == 200) {
-            		remaining -= 1;
-            		if (remaining == 0) {
-            			/* All styles set! */
-            			$scope.saveThemeButtonTitle = "Theme Set!";
-            		}
-            	}
-            })
+            postdata.colors.push({
+                "what" : name,
+                "color" : selected,
+            });
         }
+        $http.post("set_color/", postdata, { headers: {'Content-Type': 'application/json'} }).then(function(arg) {
+            if (arg.status == 200) {
+            	$scope.saveThemeButtonTitle = "Theme Set!";
+            }
+        })
     };
 
-    $scope.getCurrentTheme();
+    $scope.getThemes();
 });
 
 controllers.controller("promptController", function($scope, $http) {
@@ -198,6 +220,7 @@ controllers.controller("promptController", function($scope, $http) {
 
             // Update attributes of current prompt and select it
             $scope.samplePrompts[0].demo = selectedPrompt.demo;
+            $scope.samplePrompts[0].right = selectedPrompt.right;
             $scope.samplePrompts[0].function = selectedPrompt.function;
             $scope.samplePrompts[0].font_size = selectedPrompt.font_size;
             $scope.selectedPrompt = $scope.samplePrompts[0];
@@ -232,16 +255,16 @@ controllers.controller("functionsController", function($scope, $http) {
 
     $scope.cleanupFishFunction = function (contents) {
         /* Replace leading tabs and groups of four spaces at the beginning of a line with two spaces. */
-        lines = contents ? contents.split('\n') : [];
-        rx = /^[\t ]+/
+        var lines = contents ? contents.split('\n') : [];
+        var rx = /^[\t ]+/
         for (var i=0; i < lines.length; i++) {
-            line = lines[i]
+            var line = lines[i]
             /* Get leading tabs and spaces */
-            whitespace_arr = rx.exec(line)
+            var whitespace_arr = rx.exec(line)
             if (whitespace_arr) {
                 /* Replace four spaces with two spaces, and tabs with two spaces */
                 var whitespace = whitespace_arr[0]
-                new_whitespace = whitespace.replace(/(    )|(\t)/g, '  ')
+                var new_whitespace = whitespace.replace(/(    )|(\t)/g, '  ')
                 lines[i] = new_whitespace + line.slice(whitespace.length)
             }
         }
@@ -372,49 +395,3 @@ controllers.controller("bindingsController", function($scope, $http) {
 
     $scope.fetchBindings();
 });
-
-controllers.controller("abbreviationsController", function($scope, $http) {
-    $scope.abbreviations = [];
-    $scope.addBlank = function() {
-        // Add blank entry if it is missing
-        hasBlank = {hasBlank: false}
-        angular.forEach($scope.abbreviations, function(value, key) {
-            if (value.phrase === "" && value.word === "") {
-                this.hasBlank = true;
-            }
-        }, hasBlank);
-        if (!$scope.abbreviations) $scope.abbreviations = [];
-        if (! hasBlank.hasBlank) {
-            $scope.abbreviations.push({phrase: "", word: "", editable: true})
-        }
-    }
-    $scope.fetchAbbreviations = function() {
-        $http.get("abbreviations/").then(function(arg) {
-            $scope.abbreviations = arg.data;
-            $scope.addBlank();
-    })};
-
-    $scope.editAbbreviation = function(abbreviation) {
-        abbreviation.editable = true;
-    }
-
-    $scope.saveAbbreviation = function(abbreviation) {
-        if (abbreviation.word && abbreviation.phrase) {
-            $http.post("save_abbreviation/", abbreviation).then(function(arg) {
-                abbreviation.editable = false;
-                $scope.addBlank();
-            });
-        }
-    };
-
-    $scope.removeAbbreviation = function(abbreviation) {
-        if (abbreviation.word) {
-            $http.post("remove_abbreviation/", abbreviation).then(function(arg) {
-                $scope.abbreviations.splice($scope.abbreviations.indexOf(abbreviation), 1);
-                $scope.addBlank();
-            });
-        }
-    };
-
-    $scope.fetchAbbreviations();
-});
